/*
 * Copyright (C) Trimble 2021 ...
 *
 * ?????
 *
*/
//******************************************************************************
// Version history:
// 1.0 07/17             Initial version. (B. Miljkovic)
//
//******************************************************************************

#include <gpiod.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "gpiod_if.h"

#ifndef	CONSUMER
#define	CONSUMER	"CPU"
#endif

// Global storage
    signal_atributes mcu_bsl_en;
    signal_atributes mcu_reset;
    signal_atributes cpu_led;

//****************************************************************************
//                      LOCAL FUNCTION DEFINITIONS                          
//*****************************************************************************
//! \brief Decode gpio signal_name into chip_name and chip line_offset, and 
//!        configure the line.
//!
//! \param signal           - signal_name code
//! \param line_config      - 0 input, 1 output-low, 2 output-high
//*****************************************************************************
void GPIO_IF_line_config(const char *signal_name)
{
    struct gpiod_chip *chip_h;
	struct gpiod_line *chip_line_h;
    char *chip_name_buf;
    unsigned int chip_name_size;
    unsigned int line_offset;
    char chip_name[32];
    int ret;

    // Decode chip_name & line_offset from signal_name
	ret = gpiod_ctxless_find_line(signal_name, chip_name, sizeof(chip_name), &line_offset);
	if (ret == -1) {
		perror("Error while decoding signal name.\n");
        return;
	}
	else if (ret == 0)
	{
		perror("Signal name does not exist.\n");
        return;
	}
 
    // Get the chip_handle from the chip name
    chip_h = gpiod_chip_open_by_name(chip_name);
    if (!chip_h) {
		perror("Open chip by name failed\n");
		return;
	}
    // Get the line_handle from chip_handle & line_offset
	chip_line_h = gpiod_chip_get_line(chip_h, line_offset);
	if (!chip_line_h) {
		perror("Get chip line failed\n");
		gpiod_chip_close(chip_h);
        return;
    }
    // For each signal_name store chip_name, line_offset & line_handle
    // and configure the signal lines as output
    if (strcmp(signal_name, "MCU_BSL_EN") == 0) {
        // Store signal line atributes
        strcpy(mcu_bsl_en.chip_name_store, chip_name);
        mcu_bsl_en.line_offset = line_offset;
        mcu_bsl_en.chip_line_h = chip_line_h;
        // Reserve the chip line, setting it as an output
        ret = gpiod_line_request_output(chip_line_h, CONSUMER, MCU_BSL_EN_INIT);
        if (ret < 0) {
            goto bail_out;
        }
        // Debug print
        printf("\n\n**** Signal: %s configured ****\n", signal_name);
        printf("**** Chip name: %s \n", mcu_bsl_en.chip_name_store);
        printf("**** Line offset: %d \n", mcu_bsl_en.line_offset);
        printf("**** Chip line handle: %p \n", mcu_bsl_en.chip_line_h);
    }
    else if (strcmp(signal_name, "MCU_RESET") == 0) {
       strcpy(mcu_reset.chip_name_store, chip_name);
        mcu_reset.line_offset = line_offset;
        mcu_reset.chip_line_h = chip_line_h;
        // Reserve the chip line, setting it as an output
        ret = gpiod_line_request_output(chip_line_h, CONSUMER, MCU_RESET_INIT);
        if (ret < 0) {
            goto bail_out;
        }
        // Debug print
        printf("\n\n**** Signal: %s configured ****\n", signal_name);
        printf("**** Chip name: %s \n", mcu_reset.chip_name_store);
        printf("**** Line offset: %d \n", mcu_reset.line_offset);
        printf("**** Chip line handle: %p \n", mcu_reset.chip_line_h);
    }
    else if (strcmp(signal_name, "CPU_LED") == 0) {
        strcpy(cpu_led.chip_name_store, chip_name);
        cpu_led.line_offset = line_offset;
        cpu_led.chip_line_h = chip_line_h;
        // Reserve the chip line, setting it as an output
        ret = gpiod_line_request_output(chip_line_h, CONSUMER, 0);
        if (ret < 0) {
            goto bail_out;
        }
        // Debug print
        printf("\n\n**** Signal: %s configured ****\n", signal_name);
        printf("**** Chip name: %s \n", cpu_led.chip_name_store);
        printf("**** Line offset: %d \n", cpu_led.line_offset);
        printf("**** Chip line handle: %p \n", cpu_led.chip_line_h);        
    }
    else printf("\n\n^^^^ No signal %s found ^^^^\n", signal_name);

    //usleep(6000000);
    return;

bail_out:
    perror("Line configuration failed\n");
    gpiod_line_release(chip_line_h);
    gpiod_chip_close(chip_h);
    return;
}


//*****************************************************************************
//
//! \brief  Releases the lines 
//!
//! \param  chip_line_h - The chip line handle
//!
//! \return None
//
//*****************************************************************************
void GPIO_IF_release_lines()
{   
    // Release the lines used
    gpiod_line_release (cpu_led.chip_line_h);
    gpiod_line_release (mcu_bsl_en.chip_line_h);
    gpiod_line_release (mcu_reset.chip_line_h);
}
//*****************************************************************************
//
//! \brief  Set output gpio line high
//!
//! \param  chip_h - Pointer to the chip designator string
//! \param  chip_line - The chip line number
//!
//! \return None
//
//*****************************************************************************
void GPIO_IF_setOutputValueOnPin(struct gpiod_line *chip_line_h, int output_value)
{ 
    int ret;

    // Set the chip line output high
   	ret = gpiod_line_set_value(chip_line_h, output_value);
	if (ret < 0) {
		perror("Set line output failed\n");
    }
}


//*****************************************************************************
//
//!  \brief Read the current gpio line status
//!
//!  \param  chip_line_h - The chip line handle
//!
//!  \return 1: HIGH, 0: LOW, -1
//
//*****************************************************************************
int GPIO_IF_getInputValueFromPin(struct gpiod_line *chip_line_h)
{
    int ret;
    
    // Get the chip line input value
   	ret = gpiod_line_get_value(chip_line_h);
	if (ret < 0) {
		perror("Read line input failed\n");
	}
    return ret;
}


//*****************************************************************************
//
//! \brief  Toggle the gpio line state
//!
//! \param  chip_line_h - The chip line handle
//!
//! \return none
//!
//
//*****************************************************************************
void GPIO_IF_toggleOutputOnPin(struct gpiod_line *chip_line_h)
{
    if(GPIO_IF_getInputValueFromPin(chip_line_h) == HIGH)
    {
        GPIO_IF_setOutputValueOnPin(chip_line_h, LOW);
    }
    else
    {
        GPIO_IF_setOutputValueOnPin(chip_line_h, HIGH);
    }
}
//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
