/*
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

//******************************************************************************
// Version history:
// 1.0 07/17             Initial version. (Nima Eskandari)
// 1.1 07/17             Added Comments. (Nima Eskandari)
//----------------------------------------------------------------------------
//   Designed 2017 by Texas Instruments
//
//   Nima Eskandari
//   Texas Instruments Inc.
//   August 2017
//   Built with CCS Version: Code Composer Studio v7
//******************************************************************************

#include <stdio.h>
#include <stdbool.h>
#include <stddef.h>
#include <termios.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <gpiod.h>
#include <time.h>

#include "uart_if.h"
#include "bsl.h"
#include "config.h"
#include "gpiod_if.h"
#include "utils.h"

//*****************************************************************************
// MSP430 Image to use ********************************************************
//*****************************************************************************

#include "msp430f2274_img.h"

//*****************************************************************************
// Program the MSP430 with the specified image ********************************
//*****************************************************************************
bool ProgramMSP430()
{
	bool numberOfErrors = 0;
	bool result = true;
	uint16_t section = 0;
	uint8_t * flashStartOfData = 0;

	for (section = 0; section < flash_sections; section++)
	{
        printf("\n\n****Section: %d****\n\n", section);
        printf("Address: 0x%X\n", flash_address[section]);
        result = EraseSegment(flash_address[section]);
        if (!result)
        {
        	printf("Erase failed\n");
        	return false;
        }
        else
        {
        	printf("Erase successful\n");
        }

	}

	for (section = 0; section < flash_sections; section++)
	{
        printf("\n\n****Section: %d****\n\n", section);
        printf("Address: 0x%X\n", flash_address[section]);
        if (section == 0)
        {
            flashStartOfData = flash;
        }
        else
        {
            flashStartOfData = flashStartOfData + flash_length_of_sections[section - 1];
        }

        result = EraseCheck(flash_address[section], flash_length_of_sections[section]);
        if (!result)
        {
        	printf("Erase check failed\n");
        	return false;
        }
        else
        {
        	printf("Erase check successful\n");
        }

        result = WriteLargeDataToMemory(flash_address[section], flash_length_of_sections[section], flashStartOfData);
        if (!result)
        {
        	printf("Write Large Data To Memory failed\n");
        	return false;
        }
        else
        {
        	printf("Write Large Data To Memory successful\n");
        }

	}

	if (numberOfErrors != 0)
	{
		return false;
	}
	return true;
}

#define MAX_RETRY	5

int main()
{
	bool result = true;
	uint8_t status;
	
	printf("\n\n******** MSP430 BSL Program *****\n"); // debug

	// PinMuxConfig();	// Do not modify pin muxing as the CPU bootloader has
						// already done that.


	printf("Configuring & initialising the signals\n");
	GPIO_IF_line_config("CPU_LED");
	GPIO_IF_line_config("MCU_BSL_EN");
	GPIO_IF_line_config("MCU_RESET");


	uint8_t retry = MAX_RETRY;
	uint8_t first_pass = 0;
	int time_Delay;
	struct timespec t_Ref;
	while (retry)
	{

		printf("\n\n***** Attempt %d *****\n\n", MAX_RETRY - retry + 1);
		// Configure and initialise the signals

		if (first_pass == 1)
		{
			UART_Close();
		}

		Reset();
		printf("Device Reset\n");

		printf("Initialsing UART4\n");
		if ( !UART_Init(INIT_BAUD_RATE) )
		{
			printf("UART initialisation failed!!!\n");
			return 1;
		}

		// UART_ReadByteWithTimeout(0, &status);  TOBE removed

		BSLEntrySequence();
/*
		// Change baudrate
		printf("Change baud rate!\n");
		if ( !Change_Baud_Rate() )
		{
			printf("Change baud rate failed!!!\n");
			return 1;
		}

		printf("Re-initialsing UART4\n");
		UART_Close();
		if ( !UART_Init(MAX_BAUD_RATE) )
		{
			printf("UART initialisation failed!!!\n");
			return 1;
		}
*/
		first_pass = 1;

		// Get BSL version
		printf("Get BSL version\n");
		result = Tx_Bsl_Version();
		MsDelay(200);
		printf("\nMSP430 programming succeeded\n");
		Reset();
		printf("Device is reset\n");
		UART_Close();
		GPIO_IF_release_lines();
		return 0;

		// First mass-erase the device
		printf("Mass erasing device\n");
		result = Mass_Erase();
		if (!result)
		{
			printf("Mass erase failed\n");
			retry --;
			continue;
		}
		// Send default password
		printf("Writing password\n");
		result = WritePasswordDefault();
		if (!result)
		{
			printf("Writing password failed\n");
			retry --;
			continue;
		}
		printf("Writing Password Succeeded\n");

		uint8_t deviceIDBytes[2] = {0};
		printf("Reading Device ID\n");
		result = ReadMemory(0x0FF0, 2, deviceIDBytes);
		if (!result)
		{
			printf("Device ID reading failed\n");
			retry --;
			continue;
		}
		else
		{
			uint16_t deviceID = deviceIDBytes[0] << 8 | deviceIDBytes[1];
			printf("Device ID: 0x%X\n", deviceID);
		}

		printf("Programing MSP 430 Device\n");
		result = ProgramMSP430();
		if (!result)
		{
			printf("\nMSP430 programming failed\n");
			retry --;
			continue;
		}
		printf("\nMSP430 programming succeeded\n");
		Reset();
		printf("Device is reset\n");
		UART_Close();
		GPIO_IF_release_lines();
		return 0;
	}
	printf("Max retries exceeded\n");
	Reset();
	printf("Device is reset\n");
	UART_Close();
	GPIO_IF_release_lines();
	return 1;
}


