/*
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

//******************************************************************************
// Version history:
// 1.0 07/17             Initial version. (Nima Eskandari)
// 1.1 07/17             Added Comments. (Nima Eskandari)
//----------------------------------------------------------------------------
//   Designed 2017 by Texas Instruments
//
//   Nima Eskandari
//   Texas Instruments Inc.
//   August 2017
//   Built with CCS Version: Code Composer Studio v7
//******************************************************************************

#include <stdio.h>
#include <stdbool.h>
#include <stddef.h>
#include <termios.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>

#include "uart_if.h"
#include "config.h"
#include "utils.h"

int fd;
struct termios uart4, old;
uint8_t rec[1024];


//*****************************************************************************
// Initialize UART ************************************************************
// Opens the serial port (UART communication) *********************************
// 
// The function first sends the Change Baud command to device using the default
// 9600 baudrate, asking the device BSL to use the maximal baud rate (38400) 
// baud.
// After receiving the ACK and a minimal wait, the function sets the new baud
// rate.
//		
// If failure occurs, returns 1
//*****************************************************************************
bool UART_Init(uint8_t baud_rate)
{
	char buf[30] = "/dev/ttyS3";
	int flags;

	fd = open(buf, O_RDWR | O_NOCTTY);
	if (fd < 0)
	{
		printf("UART port failed to open!\n");
		return false;
	}
		
	flags = fcntl(fd, F_GETFL, 0);
	fcntl(fd, F_SETFL, flags | O_NONBLOCK);

	tcgetattr(fd, &old);
	bzero(&uart4, sizeof(uart4));
	
	if (baud_rate == INIT_BAUD_RATE)
	{
		uart4.c_cflag |= B9600;
	}
	else
	{
		uart4.c_cflag |= B38400;
	}

	uart4.c_cflag = B9600 | CS8 | CLOCAL | CREAD | PARENB;
	uart4.c_iflag = IGNPAR | ICRNL;
	uart4.c_oflag = 0;
	uart4.c_lflag = 0;

	uart4.c_cc[VTIME] = 0;
	uart4.c_cc[VMIN] = 1;
	tcsetattr(fd, TCSANOW, &uart4);
	// wait for any initital data and/or BSL clock setting before flushing 
	// buffers
	MsDelay(10);
	tcflush(fd, TCIOFLUSH);
	return true;
}

//*****************************************************************************
// Send byte through UART *****************************************************
// byte: the value of the byte to write ***************************************
//*****************************************************************************
void UART_SendByte(uint8_t byte)
{
	int ret;
	ret = write(fd, &byte, 1);
}

//*****************************************************************************
// Send byte array through UART ***********************************************
// byteArray: the array of the bytes to write *********************************
// size: the size of the data array *******************************************
//*****************************************************************************
void UART_SendByteArray(uint8_t * byteArray, uint16_t size)
{
	int ret;
	ret = write(fd, byteArray, size);
}

//*****************************************************************************
// Read byte through UART *****************************************************
//*****************************************************************************
uint8_t UART_ReadByte()
{
	while (1)
	{
		if (read(fd, &rec, 1) > 0)
		{
			return rec[0];
		}
	}
}

//*****************************************************************************
// Read byte through UART with the specified timeout **************************
// timeout: number of milliseconds before timeout *****************************
// error: data containor for error (whether the communication timed out) ******
//*****************************************************************************
uint8_t UART_ReadByteWithTimeout(uint16_t timeout, uint8_t * error)
{
	struct timespec t1;

	clock_gettime(CLOCK_MONOTONIC, &t1);;
#if DEBUG
		printf("Reading One Byte with timeout\n");
#endif
	while (1)
	{
		if (read(fd, &rec, 1) > 0)
		{
			*error = UART_Successful;
#if DEBUG
			printf("Read One Byte: 0x%x\n", rec[0]);
#endif
			return rec[0];
		}

		if ( Get_mS_Time_Diff(t1) > timeout )
		{
			*error = UART_Timeout;
#if DEBUG
			printf("Byte Reding Timeout!\n");
#endif
			return 0xFF;
		}
	}
}

//*****************************************************************************
// Read byte array through UART ***********************************************
// byteArray: The data read from UART is stored in this array *****************
// size: the size of the data to read *****************************************
//*****************************************************************************
void UART_ReadByteArray(uint8_t * byteArray, uint16_t size)
{
	int index = 0;
#if DEBUG
			printf("Receiving Data: 0x%x\n", rec[0]);
#endif
	while (size)
	{
		if (read(fd, &rec, 1) > 0)
		{
#if DEBUG
			printf("0x%x\n", rec[0]);
#endif
			byteArray[index++] = rec[0];
			size--;
		}
	}
#if DEBUG
			printf("Done Receiving Data\n");
#endif
}

//*****************************************************************************
// Read byte array through UART ***********************************************
// byteArray: The data read from UART is stored in this array *****************
// size: the size of the data to read *****************************************
// timeout: number of seconds before timeout **********************************
// error: data containor for error (whether the communication timed out) ******
//*****************************************************************************
void UART_ReadByteArrayWithTimeout(uint8_t * byteArray, uint16_t size, uint32_t timeout, uint8_t * error)
{
	struct timespec t1;
	int index = 0;

	clock_gettime(CLOCK_MONOTONIC, &t1);
#if DEBUG
			printf("Reading Data With Timeout: 0x%x\n", rec[0]);
#endif
	while (size)
	{
		if (read(fd, &rec, 1) > 0)
		{
#if DEBUG
			printf("0x%x\n", rec[0]);
#endif
			byteArray[index++] = rec[0];
			size--;
		}

		if ( Get_mS_Time_Diff(t1) > timeout )
		{
			*error = UART_Timeout;
#if DEBUG
			printf("Byte Array Reading Timeout\n");
#endif
			return;
		}

	}
#if DEBUG
			printf("Done Reading Data\n");
#endif
}

//*****************************************************************************
// Close the serial port and end the UART communication ***********************
//*****************************************************************************
void UART_Close()
{
	printf("Closing UART4\n");
	close(fd);
}